Attribute VB_Name = "MpqLib"
Private Const MAX_PATH                 As Long = 260

'// Values for MpqCreateArchive
Public Const MIN_HASH_TABLE            As Long = &H2
Public Const MAX_HASH_TABLE            As Long = &H40000
Public Const MIN_BLOCK_SIZE            As Long = &H1
Public Const MAX_BLOCK_SIZE            As Long = &HF

'// Values for MpqOpenArchive
Public Const MPQ_OPEN_NORMAL           As Long = &H0        '// Open normal archives

'// Values for MpqOpenFile
Public Const MPQ_OPEN_INTERNAL         As Long = &H0        '// Open files inside MPQ archive
Public Const MPQ_OPEN_LOCAL            As Long = -1         '// Open local files

'// Flags for MpqAddFile
Public Const MPQ_FILE_COMPRESS_PKWARE  As Long = &H100      '// PKWARE DCL compression
Public Const MPQ_FILE_COMPRESS_MULTI   As Long = &H200      '// Multiple compressions
Public Const MPQ_FILE_COMPRESSED       As Long = &HFF00     '// Compressed file(compatible with future compression styles)
Public Const MPQ_FILE_ENCRYPTED        As Long = &H10000    '// File is encrypted
Public Const MPQ_FILE_SEEDFIXED        As Long = &H20000    '// Encrypt seed is fixed
Public Const MPQ_FILE_EXISTS           As Long = &H80000000 '// File exists, will reset after removing the file
Public Const MPQ_FILE_REPLACEEXISTING  As Long = &H80000000 '// Replace existing files (used in MpqAddFile)

'// Constants for MpqAddWave
Public Const MPQ_WAVE_QUALITY_HIGH     As Long = &H0        '// Best quality with worst compression
Public Const MPQ_WAVE_QUALITY_MEDIUM   As Long = &H1        '// Medium quality with normal compression
Public Const MPQ_WAVE_QUALITY_LOW      As Long = &H2        '// Low quality with best compression

'// Constants for MpqGetFileInfo
'// For Archive
Public Const MPQ_INFO_ARCHIVE_SIZE     As Long = &H1        '// MPQ size (value from header, which means exclude the extra part before MPQ header)
Public Const MPQ_INFO_HASH_TABLE_SIZE  As Long = &H2        '// Count of hash table entries
Public Const MPQ_INFO_BLOCK_TABLE_SIZE As Long = &H3        '// Count of entries in the block table
Public Const MPQ_INFO_BLOCK_SIZE       As Long = &H4        '// Size of file block (in bytes)
Public Const MPQ_INFO_FILE_COUNT       As Long = &H5        '// Count of files in archive
'// For File
Public Const MPQ_INFO_HASH_INDEX       As Long = &H6        '// Hash index of the file in MPQ
Public Const MPQ_INFO_LOCALE_ID        As Long = &H7        '// Locale ID of the file
Public Const MPQ_INFO_BLOCK_INDEX      As Long = &H8        '// Index in Block Table of the file
Public Const MPQ_INFO_HASH1            As Long = &H9        '// First hash code of the file
Public Const MPQ_INFO_HASH2            As Long = &H10       '// Second hash code of the file
Public Const MPQ_INFO_FILE_SIZE        As Long = &H11       '// Original file size
Public Const MPQ_INFO_COMPRESSED_SIZE  As Long = &H12       '// Compressed file size
Public Const MPQ_INFO_FLAGS            As Long = &H13       '// File flags
Public Const MPQ_INFO_SEED             As Long = &H14       '// Encrypted file seed
Public Const MPQ_INFO_UNFIXED_SEED     As Long = &H15       '// Unfixed seed of encrypted files
Public Const MPQ_INFO_OFFSET           As Long = &H16       '// File offset in MPQ archive

'// Values for compact callback
Public Const CCB_CHECKING_FILES        As Long = &H1        '// Checking archive : (dwParam1 = current, dwParam2 = total)
Public Const CCB_COMPACTING_FILES      As Long = &H2        '// Compacting archive (dwParam1 = current, dwParam2 = total)
Public Const CCB_CLOSING_ARCHIVE       As Long = &H3        '// Closing archive : No params used

Public Const MPQLIB_VERSION            As Long = &H1020101
Public Const MPQLIB_VER_STR            As String = "1.2.1.1" '// Current version of MpqLib

'// Constants for MpqCreateArchive
Public Const CREATE_NEW                As Long = &H1
Public Const CREATE_ALWAYS             As Long = &H2
Public Const OPEN_EXISTING             As Long = &H3
Public Const OPEN_ALWAYS               As Long = &H4

'//-----------------------------------------------------------------------------
'// Structures
Public Type T_MPQ_FIND_DATA
    szFullPath      As String * MAX_PATH   '// Full path name
    szFileName      As Long                '// Pointer to file name part
    dwFileSize      As Long                '// Original file size
    dwCompSize      As Long                '// Compressed size
    dwFlags         As Long                '// File flags
    dwBlockIndex    As Long                '// Block index of the file
    lcLocale        As Long 'LCID          '// Locale ID
End Type
'//-----------------------------------------------------------------------------
'// Functions in MpqLib

'typedef void  (WINAPI * COMPACTCB)(DWORD dwProcType, DWORD dwParam1, DWORD dwParam2, void * lpData);

'// Archive opening/closing
Declare Function MpqOpenArchive Lib "MpqLib.dll" Alias "A3" (ByVal szMpqName As String, ByVal dwPriority As Long, ByVal dwFlags As Long, ByRef phMPQ As Long, Optional ByVal bNoListFile As Boolean = False) As Boolean
'BOOL  WINAPI MpqOpenArchive(const char * szMpqName, DWORD dwPriority, DWORD dwFlags, HANDLE * phMPQ, BOOL bNoListFile = false);
'/*
'  MpqOpenArchive - Open an archive, readonly
'   return TRUE if successfully opened the archive, otherwise return FALSE.
'    szMpqName  : MPQ FileName
'    dwPriority : Priority for searching files in multiple archives, not used currently
'    dwFlags    : Open flags, not used currently, please use MPQ_OPEN_NORMAL
'    phMPQ      : Pointer to retrieve MPQ handle
'    bNoListFile: Do not parse listfile for list on opening archive
'*/

Declare Function MpqCreateArchive Lib "MpqLib.dll" Alias "B4" (ByVal szMpqName As String, ByVal dwCreationDisposition As Long, ByVal dwHashTableSize As Long, ByRef phMPQ As Long, Optional ByVal dwBlockSize As Long = 3, Optional ByVal bNoListFile As Boolean = False) As Boolean
'BOOL  WINAPI MpqCreateArchive(const char * szMpqName, DWORD dwCreationDisposition, DWORD dwHashTableSize, HANDLE * phMPQ, DWORD dwBlockSize = 3, BOOL bNoListFile = false);
'/*
'  MpqCreateArchive - Open or Create an archive
'   return TRUE if successfully opened/created the archive, otherwise return FALSE.
'   szMpqName             : MPQ FileName
'    dwCreationDisposition: can be CREATE_NEW, CREATE_ALWAYS, OPEN_EXISTING or OPEN_ALWAYS, check Windows API CreateFile for more information
'    dwHashTableSize      : Hashtable size, which indicates the maximum count of files allowed, range:  As Long = &H2- As Long = &H40000, should be power of 2
'    phMPQ                : Pointer to retrieve MPQ handle
'    dwBlockSize          : Block size of file compression, range:  As Long = &H01- As Long = &H0F, the real block size is ( As Long = &H200 * 2 ^ n)
'    bNoListFile          : Do not parse listfile for list on opening/creating archive and do not save it on closing
'*/

Declare Function MpqCloseArchive Lib "MpqLib.dll" Alias "A4" (ByVal phMPQ As Long) As Boolean
'BOOL  WINAPI MpqCloseArchive(HANDLE hMPQ);
'/*
'  MpqCloseArchive - Close an archive, works on handles retrieved by both MpqOpenArchive and MpqCreateArchive
'   return TRUE if successfully closed the archive, otherwise return FALSE.
'    hMPQ : handle of the MPQ archive to close
'*/

'// File opening/closing
Declare Function MpqOpenFile Lib "MpqLib.dll" Alias "A5" (ByVal hMPQ As Long, ByVal szFileName As String, ByVal dwSearchScope As Long, ByRef phFile As Long) As Boolean
'BOOL  WINAPI MpqOpenFile(HANDLE hMPQ, const char * szFileName, DWORD dwSearchScope, HANDLE * phFile);
'/*
'  MpqOpenFile - Open a file
'   return TRUE if successfully opened the file, otherwise return FALSE.
'    hMPQ         : handle of the MPQ archive for opening file
'    szFileName   : Filename or Hashtable Index(use typecast to convert a DWORD to const char *) of file to open
'    dwSearchScope: MPQ_OPEN_INTERNAL - Open files in mpq archive
'                   MPQ_OPEN_LOCAL    - Open files on local hdd
'    phFile       : Pointer to retrieve file handle
'*/

Declare Function MpqCloseFile Lib "MpqLib.dll" Alias "A6" (ByVal hFile As Long) As Boolean
'BOOL  WINAPI MpqCloseFile(HANDLE hFile);
'/*
'  MpqCloseFile - Close a file, works on handles retrieved by MpqOpenFile
'   return TRUE if successfully closed the file, otherwise return FALSE.
'    hFile : handle of the file to close
'*/

'// File I/O
Declare Function MpqGetFileOffset Lib "MpqLib.dll" Alias "A7" (ByVal hFile As Long, Optional ByRef pdwFileOffsetHigh As Long = 0) As Long
'DWORD WINAPI MpqGetFileOffset(HANDLE hFile, DWORD * pdwFileOffsetHigh = NULL);
'/*
'  MpqGetFileOffset - Get file offset in the MPQ archive
'   return the offset of the file, otherwise return  As Long = &HFFFFFFFF
'    hFile            : File Handle
'    pdwFileOffsetHigh: Stores upper 32bits of the offset if valid
'*/

Declare Function MpqGetFileSize Lib "MpqLib.dll" Alias "A8" (ByVal hFile As Long, Optional ByRef pdwFileSizeHigh As Long = 0) As Long
'DWORD WINAPI MpqGetFileSize(HANDLE hFile, DWORD * pdwFileSizeHigh = NULL);
'/*
'  MpqGetFileSize - Get original file size
'   return the original size of the file, otherwise return  As Long = &HFFFFFFFF
'    hFile          : File Handle
'    pdwFileSizeHigh: Stores upper 32bits of the size if valid
'*/

Declare Function MpqSetFilePointer Lib "MpqLib.dll" Alias "A9" (ByVal hFile As Long, ByVal lFileOffset As Long, ByRef pdwFileOffsetHigh As Long, ByVal dwMethod As Long) As Long
'DWORD WINAPI MpqSetFilePointer(HANDLE hFile, LONG lFileOffset, LONG * pdwFileOffsetHigh, DWORD dwMethod);
'/*
'  MpqSetFilePointer - Move file stream pointer
'   return the new offset of the pointer, otherwise return  As Long = &HFFFFFFFF
'    hFile            : File Handle
'    lFileOffset      : The move offset
'    pdwFileOffsetHigh: Stores upper 32bits of the new offset if valid
'    dwMethod         : FILE_BEGIN   - Move from beginning
'                       FILE_CURRENT - Move from current position
'                       FILE_END     - Move from end of file (lFileOffset should be equal or less than zero)
'*/

Declare Function MpqReadFile Lib "MpqLib.dll" Alias "B1" (ByVal hFile As Long, ByRef lpBuffer As Any, ByVal dwToRead As Long, Optional ByVal lpOverlapped As Long = 0, Optional ByRef pdwRead As Long = 0) As Boolean
'BOOL  WINAPI MpqReadFile(HANDLE hFile, VOID * lpBuffer, DWORD dwToRead, DWORD * pdwRead = NULL, LPOVERLAPPED lpOverlapped = NULL);
'/*
'  MpqReadFile - Read data from a file
'   return TRUE if successfully read data, otherwise return FALSE
'    hFile       : File Handle
'    lpBuffer    : Buffer to store read data
'    dwToRead    : Count of bytes to read
'    pdwRead     : Stores count of read bytes if valid
'    lpOverlapped: Check Windows API ReadFile for information, do not work on internal files of archive
'*/

Declare Function MpqExtractFile Lib "MpqLib.dll" Alias "B2" (ByVal hMPQ As Long, ByVal szToExtract As String, ByVal szExtracted As String) As Boolean
'BOOL  WINAPI MpqExtractFile(HANDLE hMpq, const char * szToExtract, const char * szExtracted);
'/*
'  MpqExtractFile - Extract a file from the MPQ archive
'   return TRUE if successfully extract the file, otherwise return FALSE
'    hMPQ       : MPQ Handle
'    szToExtract: Name of the file to extract
'    szExtracted: Full path name of extracted file
'*/

Declare Function MpqAddListFile Lib "MpqLib.dll" Alias "B3" (ByVal hMPQ As Long, ByVal szListFile As String) As Boolean
'BOOL  WINAPI MpqAddListFile(HANDLE hMpq, const char * szListFile);
'/*
'  MpqAddListFile - Merge a external listfile into current MPQ archive
'   return TRUE if successfully added the listfile, otherwise return FALSE
'    hMpq       : MPQ Handle
'    szListFile : Full path name of the listfile
'*/

'// Modify archive functions
Declare Function MpqAddFile Lib "MpqLib.dll" Alias "B5" (ByVal hMPQ As Long, ByVal szFileName As String, ByVal szArchivedName As String, ByVal dwFlags As Long, ByRef pbReplaced As Boolean) As Boolean
'BOOL  WINAPI MpqAddFile(HANDLE hMPQ, const char * szFileName, const char * szArchivedName, DWORD dwFlags, BOOL * pbReplaced);
'/*
'  MpqAddFile - Add a file into the MPQ archive
'   return TRUE if successfully added the file, otherwise return FALSE
'    hMpq          : MPQ Handle
'    szFileName    : Name of file to add
'    szArchivedName: Name of archived file
'    dwFlags       : Flags of file (check constant part)
'    pbReplaced    : If valid, it retrieve whether the file is replaced or not(newly added)
'*/

Declare Function MpqAddWave Lib "MpqLib.dll" Alias "B6" (ByVal hMPQ As Long, ByVal szFileName As String, ByVal szArchivedName As String, ByVal dwFlags As Long, ByVal dwQuality As Long, Optional ByRef pbReplaced As Boolean = 0) As Boolean
'BOOL  WINAPI MpqAddWave(HANDLE hMPQ, const char * szFileName, const char * szArchivedName, DWORD dwFlags, DWORD dwQuality, BOOL * pbReplaced = NULL);
'/*
'  MpqAddWave - Add a wave file into the MPQ archive
'   return TRUE if successfully added the wave file, otherwise return FALSE
'    hMpq          : MPQ Handle
'    szFileName    : Name of file to add
'    szArchivedName: Name of archived file
'    dwFlags       : Flags of file (check constant part)
'    dwQuality     : Wave quality (check constant part)
'    pbReplaced    : If valid, it retrieve whether the file is replaced or not(newly added)
'*/

Declare Function MpqAddBuffer Lib "MpqLib.dll" Alias "C8" (ByVal hMPQ As Long, ByRef pbData As Any, ByVal dwLength As Long, ByVal szArchivedName As String, ByVal dwFlags As Long, Optional ByVal bIsWave As Boolean = False, Optional ByVal dwQuality As Long = 0, Optional ByRef pbReplaced As Boolean = 0) As Boolean
'BOOL  WINAPI MpqAddBuffer(HANDLE hMPQ, BYTE * pbData, DWORD dwLength, const char * szArchivedName, DWORD dwFlags, BOOL bIsWave = FALSE, DWORD dwQuality = 0, BOOL * pbReplaced = NULL);
'/*
'  MpqAddBuffer - Add buffer data into the MPQ archive as a file
'   return TRUE if successfully added the file, otherwise return FALSE
'    hMpq          : MPQ Handle
'    pbData        : Data Buffer
'    dwLength      : Length of data buffer
'    szArchivedName: Name of archived file
'    dwFlags       : Flags of file (check constant part)
'    bIsWave       : Indicates if the it is a wave file
'    dwQuality     : Wave quality (check constant part)
'    pbReplaced    : If valid, it retrieve whether the file is replaced or not(newly added)
'*/

Declare Function MpqRemoveFile Lib "MpqLib.dll" Alias "B7" (ByVal hMPQ As Long, ByVal szFileName As String) As Boolean
'BOOL  WINAPI MpqRemoveFile(HANDLE hMPQ, const char * szFileName);
'/*
'  MpqRemoveFile - Remove a file from the MPQ archive
'   return TRUE if successfully removed the file, otherwise return FALSE
'    hMpq      : MPQ Handle
'    szFileName: Name of file to remove
'*/

Declare Function MpqRenameFile Lib "MpqLib.dll" Alias "B8" (ByVal hMPQ As Long, ByVal szOldFileName As String, ByVal szNewFileName As String) As Boolean
'BOOL  WINAPI MpqRenameFile(HANDLE hMPQ, const char * szOldFileName, const char * szNewFileName);
'/*
'  MpqRemoveFile - Rename a file in the MPQ archive
'   return TRUE if successfully renamed the file, otherwise return FALSE
'    hMpq         : MPQ Handle
'    szOldFileName: Name of file to rename
'    szNewFileName: Name of renamed file
'*/

'// Retrieving archive/file info functions
Declare Function MpqHasFile Lib "MpqLib.dll" Alias "C1" (ByVal hMPQ As Long, ByVal szFileName As String) As Boolean
'BOOL  WINAPI MpqHasFile(HANDLE hMPQ, char * szFileName);
'/*
'  MpqHasFile - Check if a file exists in the MPQ archive
'   return TRUE if the file exists, otherwise return FALSE
'    hMpq      : MPQ Handle
'    szFileName: Name of file to check
'*/

Declare Function MpqGetFileName Lib "MpqLib.dll" Alias "C2" (ByVal hFile As Long, ByVal szFileName As String) As Boolean
'BOOL  WINAPI MpqGetFileName(HANDLE hFile, char * szFileName);
'/*
'  MpqHasFile - Retrieve filename from a file handle
'   return TRUE if the successfully retrieved, otherwise return FALSE
'    hFile     : File Handle
'    szFileName: Buffer to retrieve the filename, the size should be enough to store the string (always be MAX_PATH)
'*/

Declare Function MpqGetFileInfo Lib "MpqLib.dll" Alias "C3" (ByVal hHandle As Long, ByVal dwInfoType As Long) As Long
'DWORD WINAPI MpqGetFileInfo(HANDLE hHandle, DWORD dwInfoType);
'/*
'  MpqGetFileInfo - Retrieved archive/file info from handle
'   return info data if successfully retrieved, otherwise return  As Long = &HFFFFFFFF
'    hHandle   : MPQ or File handle
'    dwInfoType: Information type (check constant part)
'*/

'// Find file functions
Declare Function MpqFindFirstFile Lib "MpqLib.dll" Alias "C4" (ByVal hMPQ As Long, ByVal szMask As String, ByRef lpFindFileData As T_MPQ_FIND_DATA, ByVal szListFile As String) As Long
'HANDLE WINAPI MpqFindFirstFile(HANDLE hMPQ, const char * szMask, T_MPQ_FIND_DATA * lpFindFileData, const char * szListFile);
'/*
'  MpqFileFirstFile - Find first matching file in the MPQ archive
'   return search handle if successfully find a matching file, otherwise return NULL
'    hMpq          : MPQ Handle
'    szMask        : Matching mask for searching
'    lpFindFileData: Retrieve info of found file
'    szListFile    : Additional listfile for searching
'*/

Declare Function MpqFindNextFile Lib "MpqLib.dll" Alias "C5" (ByVal hFind As Long, ByRef lpFindFileData As T_MPQ_FIND_DATA) As Boolean
'BOOL   WINAPI MpqFindNextFile(HANDLE hFind, T_MPQ_FIND_DATA * lpFindFileData);
'/*
'  MpqFileNextFile - Find next matching file in the MPQ archive
'   return TRUE if successfully find a matching file, otherwise return FALSE
'    hFind         : Search Handle
'    lpFindFileData: Retrieve info of found file
'*/

Declare Function MpqFindClose Lib "MpqLib.dll" Alias "C6" (ByVal hFind As Long) As Boolean
'BOOL   WINAPI MpqFindClose(HANDLE hFind);
'/*
'  MpqFileClose - Close a search handle
'   return TRUE if successfully closed the handle, otherwise return FALSE
'    hFind : Search handle to close
'*/

'// Compact/Flush function
Declare Function MpqCompactArchive Lib "MpqLib.dll" Alias "C7" (ByVal hMPQ As Long, Optional ByVal szListFile As String = "", Optional ByRef lpCallbackData As Any) As Boolean
'BOOL  WINAPI MpqCompactArchive(HANDLE hMPQ, const char * szListFile = NULL, COMPACTCB CompactCB = NULL, void * lpCallbackData = NULL);
'/*
'  MpqCompactArchive - Compact/Flush an archive
'   return TRUE if successfully compacting the archive, otherwise return FALSE
'    hMPQ          : MPQ Handle
'    szListFile    : Additional listfile for compacting
'    CompactCB     : Compact callback function
'    lpCallBackData: lpData parameter of callback function
'*/

'// Locale functions
Declare Function MpqSetLocale Lib "MpqLib.dll" Alias "A1" (ByVal lcNewLocale As Long) As Long
'LCID  WINAPI MpqSetLocale(LCID lcNewLocale);
'/*
'  MpqSetLocale - Set current locale of the archive (will affect both read and modify functions)
'   return old locale ID
'    lcNewLocale : New Locale ID
'*/

Declare Function MpqGetLocale Lib "MpqLib.dll" Alias "A2" () As Long
'LCID  WINAPI MpqGetLocale();
'/*
'  MpqSetLocale - retrieve current locale ID
'   return current locale ID
'*/

Declare Function MpqSetFileLocale Lib "MpqLib.dll" Alias "B9" (ByVal hFile As Long, ByVal lcNewLocale As Long) As Boolean
'BOOL  WINAPI MpqSetFileLocale(HANDLE hFile, LCID lcNewLocale);
'/*
'  MpqSetLocale - Set locale of a file (only affect files in MPQ archives)
'   return TRUE if successfully set the locale, otherwise return FALSE
'    hFile       : File Handle
'    lcNewLocale : New Locale ID
'*/
