#ifndef _MPQLIB_H_
#define _MPQLIB_H_

#include <windows.h>

//-----------------------------------------------------------------------------
// Defines

// Values for MpqCreateArchive
#define MIN_HASH_TABLE           0x00002
#define MAX_HASH_TABLE           0x40000
#define MIN_BLOCK_SIZE           0x01
#define MAX_BLOCK_SIZE           0x0F

// Values for MpqOpenArchive
#define MPQ_OPEN_NORMAL          0        // Open normal archives

// Values for MpqOpenFile
#define MPQ_OPEN_INTERNAL        0          // Open files inside MPQ archive
#define MPQ_OPEN_LOCAL           (DWORD)-1  // Open local files

// Flags for MpqAddFile
#define MPQ_FILE_COMPRESS_PKWARE 0x00000100 // PKWARE DCL compression
#define MPQ_FILE_COMPRESS_MULTI  0x00000200 // Multiple compressions
#define MPQ_FILE_COMPRESSED      0x0000FF00 // Compressed file(compatible with future compression styles)
#define MPQ_FILE_ENCRYPTED       0x00010000 // File is encrypted
#define MPQ_FILE_SEEDFIXED       0x00020000 // Encrypt seed is fixed
#define MPQ_FILE_SINGLE_UNIT     0x01000000 // File is stored in a single unit
#define MPQ_FILE_EXISTS          0x80000000 // File exists, will reset after removing the file
#define MPQ_FILE_REPLACEEXISTING 0x80000000 // Replace existing files (used in MpqAddFile)

// Constants for MpqAddWave
#define MPQ_WAVE_QUALITY_HIGH        0      // Best quality with worst compression
#define MPQ_WAVE_QUALITY_MEDIUM      1      // Medium quality with normal compression
#define MPQ_WAVE_QUALITY_LOW         2      // Low quality with best compression

// Constants for MpqGetFileInfo
// For Archive
#define MPQ_INFO_ARCHIVE_SIZE      1      // MPQ size (value from header, which means exclude the extra part before MPQ header)
#define MPQ_INFO_HASH_TABLE_SIZE   2      // Count of hash table entries
#define MPQ_INFO_BLOCK_TABLE_SIZE  3      // Count of entries in the block table
#define MPQ_INFO_BLOCK_SIZE        4      // Size of file block (in bytes)
#define MPQ_INFO_FILE_COUNT        5      // Count of files in archive
// For File
#define MPQ_INFO_HASH_INDEX        6      // Hash index of the file in MPQ
#define MPQ_INFO_LOCALE_ID         7      // Locale ID of the file
#define MPQ_INFO_BLOCK_INDEX       8      // Index in Block Table of the file
#define MPQ_INFO_HASH1             9      // First hash code of the file
#define MPQ_INFO_HASH2            10      // Second hash code of the file
#define MPQ_INFO_FILE_SIZE        11      // Original file size
#define MPQ_INFO_COMPRESSED_SIZE  12      // Compressed file size
#define MPQ_INFO_FLAGS            13      // File flags
#define MPQ_INFO_SEED             14      // Encrypted file seed
#define MPQ_INFO_UNFIXED_SEED     15      // Unfixed seed of encrypted files
#define MPQ_INFO_OFFSET           16      // File offset in MPQ archive

// Values for compact callback
#define CCB_CHECKING_FILES         1      // Checking archive : (dwParam1 = current, dwParam2 = total)
#define CCB_COMPACTING_FILES       2      // Compacting archive (dwParam1 = current, dwParam2 = total)
#define CCB_CLOSING_ARCHIVE        3      // Closing archive : No params used

#define MPQLIB_VERSION    0x01020101
#define MPQLIB_VER_STR     "1.2.1.1"      // Current version of MpqLib

//-----------------------------------------------------------------------------
// Structures

struct T_MPQ_FIND_DATA
{
    char   szFullPath[MAX_PATH];        // Full path name
    char * szFileName;                  // Pointer to file name part
    DWORD  dwFileSize;                  // Original file size
    DWORD  dwCompSize;                  // Compressed size
    DWORD  dwFlags;                 // File flags
    DWORD  dwBlockIndex;                // Block index of the file
    LCID   lcLocale;                    // Locale ID
};

//-----------------------------------------------------------------------------
// Functions in MpqLib

typedef void  (WINAPI * COMPACTCB)(DWORD dwProcType, DWORD dwParam1, DWORD dwParam2, void * lpData);

#ifdef __cplusplus
extern "C" {
#endif

// Archive opening/closing
BOOL  WINAPI MpqOpenArchive(const char * szMpqName, DWORD dwPriority, DWORD dwFlags, HANDLE * phMPQ, BOOL bNoListFile
#ifdef __cplusplus
 = FALSE
#endif
);
/*
  MpqOpenArchive - Open an archive, readonly
   return TRUE if successfully opened the archive, otherwise return FALSE.
    szMpqName  : MPQ Filename
    dwPriority : Priority for searching files in multiple archives, not used currently
    dwFlags    : Open flags, not used currently, please use MPQ_OPEN_NORMAL
    phMPQ      : Pointer to retrieve MPQ handle
    bNoListFile: Do not parse listfile for list on opening archive
*/

BOOL  WINAPI MpqCreateArchive(const char * szMpqName, DWORD dwCreationDisposition, DWORD dwHashTableSize, HANDLE * phMPQ, DWORD dwBlockSize
#ifdef __cplusplus
							  = 3
#endif
							  , BOOL bNoListFile
#ifdef __cplusplus
							  = FALSE
#endif
							  );
/*
  MpqCreateArchive - Open or Create an archive
   return TRUE if successfully opened/created the archive, otherwise return FALSE.
    szMpqName            : MPQ Filename
    dwCreationDisposition: can be CREATE_NEW, CREATE_ALWAYS, OPEN_EXISTING or OPEN_ALWAYS, check Windows API CreateFile for more information
    dwHashTableSize      : Hashtable size, which indicates the maximum count of files allowed, range: 0x2-0x40000, should be power of 2
    phMPQ                : Pointer to retrieve MPQ handle
    dwBlockSize          : Block size of file compression, range: 0x01-0x0F, the real block size is (0x200 * 2 ^ n)
    bNoListFile          : Do not parse listfile for list on opening/creating archive and do not save it on closing
*/

BOOL  WINAPI MpqCloseArchive(HANDLE hMPQ);
/*
  MpqCloseArchive - Close an archive, works on handles retrieved by both MpqOpenArchive and MpqCreateArchive
   return TRUE if successfully closed the archive, otherwise return FALSE.
    hMPQ : handle of the MPQ archive to close
*/

// File opening/closing
BOOL  WINAPI MpqOpenFile(HANDLE hMPQ, const char * szFileName, DWORD dwSearchScope, HANDLE * phFile);
/*
  MpqOpenFile - Open a file
   return TRUE if successfully opened the file, otherwise return FALSE.
    hMPQ         : handle of the MPQ archive for opening file
    szFileName   : Filename or Hashtable Index(use typecast to convert a DWORD to const char *) of file to open
    dwSearchScope: MPQ_OPEN_INTERNAL - Open files in mpq archive
                   MPQ_OPEN_LOCAL    - Open files on local hdd
    phFile       : Pointer to retrieve file handle
*/

BOOL  WINAPI MpqCloseFile(HANDLE hFile);
/*
  MpqCloseFile - Close a file, works on handles retrieved by MpqOpenFile
   return TRUE if successfully closed the file, otherwise return FALSE.
    hFile : handle of the file to close
*/

// File I/O
DWORD WINAPI MpqGetFileOffset(HANDLE hFile, DWORD * pdwFileOffsetHigh
#ifdef __cplusplus
							  = NULL
#endif
							  );
/*
  MpqGetFileOffset - Get file offset in the MPQ archive
   return the offset of the file, otherwise return 0xFFFFFFFF
    hFile            : File handle
    pdwFileOffsetHigh: Stores upper 32bits of the offset if valid
*/

DWORD WINAPI MpqGetFileSize(HANDLE hFile, DWORD * pdwFileSizeHigh
#ifdef __cplusplus
							= NULL
#endif
							);
/*
  MpqGetFileSize - Get original file size
   return the original size of the file, otherwise return 0xFFFFFFFF
    hFile          : File handle
    pdwFileSizeHigh: Stores upper 32bits of the size if valid
*/

DWORD WINAPI MpqSetFilePointer(HANDLE hFile, LONG lFileOffset, LONG * pdwFileOffsetHigh, DWORD dwMethod);
/*
  MpqSetFilePointer - Move file stream pointer
   return the new offset of the pointer, otherwise return 0xFFFFFFFF
    hFile            : File handle
    lFileOffset      : The move offset
    pdwFileOffsetHigh: Stores upper 32bits of the new offset if valid
    dwMethod         : FIlE_BEGIN   - Move from beginning
                       FILE_CURRENT - Move from current position
                       FILE_END     - Move from end of file (lFileOffset should be equal or less than zero)
*/

BOOL  WINAPI MpqReadFile(HANDLE hFile, VOID * lpBuffer, DWORD dwToRead, DWORD * pdwRead
#ifdef __cplusplus
						 = NULL
#endif
						 , LPOVERLAPPED lpOverlapped
#ifdef __cplusplus
						 = NULL
#endif
						 );
/*
  MpqReadFile - Read data from a file
   return TRUE if successfully read data, otherwise return FALSE
    hFile       : File handle
    lpBuffer    : Buffer to store read data
    dwToRead    : Count of bytes to read
    pdwRead     : Stores count of read bytes if valid
    lpOverlapped: Check Windows API ReadFile for information, do not work on internal files of archive
*/

BOOL  WINAPI MpqExtractFile(HANDLE hMpq, const char * szToExtract, const char * szExtracted);
/*
  MpqExtractFile - Extract a file from the MPQ archive
   return TRUE if successfully extract the file, otherwise return FALSE
    hMpq       : MPQ handle
    szToExtract: Name of the file to extract
    szExtracted: Full path name of extracted file
*/

BOOL  WINAPI MpqAddListFile(HANDLE hMpq, const char * szListFile);
/*
  MpqAddListFile - Merge a external listfile into current MPQ archive
   return TRUE if successfully added the listfile, otherwise return FALSE
    hMpq       : MPQ handle
    szListFile : Full path name of the listfile
*/

// Modify archive functions
BOOL  WINAPI MpqAddFile(HANDLE hMPQ, const char * szFileName, const char * szArchivedName, DWORD dwFlags, BOOL * pbReplaced);
/*
  MpqAddFile - Add a file into the MPQ archive
   return TRUE if successfully added the file, otherwise return FALSE
    hMpq          : MPQ handle
    szFileName    : Name of file to add
    szArchivedName: Name of archived file
    dwFlags       : Flags of file (check constant part)
    pbReplaced    : If valid, it retrieve whether the file is replaced or not(newly added)
*/

BOOL  WINAPI MpqAddWave(HANDLE hMPQ, const char * szFileName, const char * szArchivedName, DWORD dwFlags, DWORD dwQuality, BOOL * pbReplaced
#ifdef __cplusplus
						= NULL
#endif
						);
/*
  MpqAddWave - Add a wave file into the MPQ archive
   return TRUE if successfully added the wave file, otherwise return FALSE
    hMpq          : MPQ handle
    szFileName    : Name of file to add
    szArchivedName: Name of archived file
    dwFlags       : Flags of file (check constant part)
    dwQuality     : Wave quality (check constant part)
    pbReplaced    : If valid, it retrieve whether the file is replaced or not(newly added)
*/

BOOL  WINAPI MpqAddBuffer(HANDLE hMPQ, BYTE * pbData, DWORD dwLength, const char * szArchivedName, DWORD dwFlags, BOOL bIsWave
#ifdef __cplusplus
						  = FALSE
#endif
						  , DWORD dwQuality
#ifdef __cplusplus
						  = 0
#endif
						  , BOOL * pbReplaced
#ifdef __cplusplus
						  = NULL
#endif
						  );
/*
  MpqAddBuffer - Add buffer data into the MPQ archive as a file
   return TRUE if successfully added the file, otherwise return FALSE
    hMpq          : MPQ handle
    pbData        : Data buffer
    dwLength      : Length of data buffer
    szArchivedName: Name of archived file
    dwFlags       : Flags of file (check constant part)
    bIsWave       : Indicates if the it is a wave file
    dwQuality     : Wave quality (check constant part)
    pbReplaced    : If valid, it retrieve whether the file is replaced or not(newly added)
*/

BOOL  WINAPI MpqRemoveFile(HANDLE hMPQ, const char * szFileName);
/*
  MpqRemoveFile - Remove a file from the MPQ archive
   return TRUE if successfully removed the file, otherwise return FALSE
    hMPQ      : MPQ handle
    szFileName: Name of file to remove
*/

BOOL  WINAPI MpqRenameFile(HANDLE hMPQ, const char * szOldFileName, const char * szNewFileName);
/*
  MpqRemoveFile - Rename a file in the MPQ archive
   return TRUE if successfully renamed the file, otherwise return FALSE
    hMPQ         : MPQ handle
    szOldFileName: Name of file to rename
    szNewFileName: Name of renamed file
*/

// Retrieving archive/file info functions
BOOL  WINAPI MpqHasFile(HANDLE hMPQ, char * szFileName);
/*
  MpqHasFile - Check if a file exists in the MPQ archive
   return TRUE if the file exists, otherwise return FALSE
    hMPQ      : MPQ handle
    szFileName: Name of file to check
*/

BOOL  WINAPI MpqGetFileName(HANDLE hFile, char * szFileName);
/*
  MpqHasFile - Retrieve filename from a file handle
   return TRUE if the successfully retrieved, otherwise return FALSE
    hFile     : File handle
    szFileName: Buffer to retrieve the filename, the size should be enough to store the string (always be MAX_PATH)
*/

DWORD WINAPI MpqGetFileInfo(HANDLE hHandle, DWORD dwInfoType);
/*
  MpqGetFileInfo - Retrieved archive/file info from handle
   return info data if successfully retrieved, otherwise return 0xFFFFFFFF
    hHandle   : MPQ or File handle
    dwInfoType: Information type (check constant part)
*/

// Find file functions
HANDLE WINAPI MpqFindFirstFile(HANDLE hMPQ, const char * szMask, struct T_MPQ_FIND_DATA * lpFindFileData, const char * szListFile);
/*
  MpqFileFirstFile - Find first matching file in the MPQ archive
   return search handle if successfully find a matching file, otherwise return NULL
    hMPQ          : MPQ handle
    szMask        : Matching mask for searching
    lpFindFileData: Retrieve info of found file
    szListFile    : Additional listfile for searching
*/

BOOL   WINAPI MpqFindNextFile(HANDLE hFind, struct T_MPQ_FIND_DATA * lpFindFileData);
/*
  MpqFileNextFile - Find next matching file in the MPQ archive
   return TRUE if successfully find a matching file, otherwise return FALSE
    hFind         : Search handle
    lpFindFileData: Retrieve info of found file
*/

BOOL   WINAPI MpqFindClose(HANDLE hFind);
/*
  MpqFileClose - Close a search handle
   return TRUE if successfully closed the handle, otherwise return FALSE
    hFind : Search handle to close
*/

// Compact/Flush function
BOOL  WINAPI MpqCompactArchive(HANDLE hMPQ, const char * szListFile
#ifdef __cplusplus
							   = NULL
#endif
							   , COMPACTCB CompactCB
#ifdef __cplusplus
							   = NULL
#endif
							   , void * lpCallbackData
#ifdef __cplusplus
							   = NULL
#endif
							   );
/*
  MpqCompactArchive - Compact/Flush an archive
   return TRUE if successfully compacting the archive, otherwise return FALSE
    hMPQ          : MPQ handle
    szListFile    : Additional listfile for compacting
    CompactCB     : Compact callback function
    lpCallBackData: lpData parameter of callback function
*/

// Locale functions
LCID  WINAPI MpqSetLocale(LCID lcNewLocale);
/*
  MpqSetLocale - Set current locale of the archive (will affect both read and modify functions)
   return old locale ID
    lcNewLocale : New Locale ID
*/

LCID  WINAPI MpqGetLocale();
/*
  MpqSetLocale - retrieve current locale ID
   return current locale ID
*/

BOOL  WINAPI MpqSetFileLocale(HANDLE hFile, LCID lcNewLocale);
/*
  MpqSetLocale - Set locale of a file (only affect files in MPQ archives)
   return TRUE if successfully set the locale, otherwise return FALSE
    hFile       : File handle
    lcNewLocale : New Locale ID
*/

#ifdef __cplusplus__
}
#endif

#endif  // _MPQLIB_H_
