unit MpqLib;

interface
uses Windows;

const
  MIN_HASH_TABLE           =  $00002;
  MAX_HASH_TABLE           =  $40000;
  MIN_BLOCK_SIZE           =  $01;
  MAX_BLOCK_SIZE           =  $0F;

  MPQ_FILE_COMPRESS_PKWARE =  $00000100; // PKWARE DCL compression
  MPQ_FILE_COMPRESS_MULTI  =  $00000200; // Multiple compressions
  MPQ_FILE_COMPRESSED      =  $0000FF00; // Compressed file(compatible with future compression styles)
  MPQ_FILE_ENCRYPTED       =  $00010000; // File is encrypted
  MPQ_FILE_SEEDFIXED       =  $00020000; // Encrypt seed is fixed
  MPQ_FILE_EXISTS          =  $80000000; // File exists, will reset after removing the file
  MPQ_FILE_REPLACEEXISTING =  $80000000; // Replace existing files (used in MpqAddFile)  MPQ_FILE_COMPRESS_PKWARE =  $00000100;

  MPQ_WAVE_QUALITY_HIGH    =  0;      // Best quality with worst compression
  MPQ_WAVE_QUALITY_MEDIUM  =  1;      // Medium quality with normal compression
  MPQ_WAVE_QUALITY_LOW     =  2;      // Low quality with best compression

  MPQ_INFO_ARCHIVE_SIZE    =  1;      // MPQ size (value from header, which means exclude the extra part before MPQ header)
  MPQ_INFO_HASH_TABLE_SIZE =  2;      // Count of hash table entries
  MPQ_INFO_BLOCK_TABLE_SIZE=  3;      // Count of entries in the block table
  MPQ_INFO_BLOCK_SIZE      =  4;      // Size of file block (in bytes)
  MPQ_INFO_FILE_COUNT      =  5;      // Count of files in archive

  MPQ_INFO_HASH_INDEX      =  6;      // Hash index of the file in MPQ
  MPQ_INFO_LOCALE_ID       =  7;      // Locale ID of the file
  MPQ_INFO_BLOCK_INDEX     =  8;      // Index in Block Table of the file
  MPQ_INFO_HASH1           =  9;      // First hash code of the file
  MPQ_INFO_HASH2           = 10;      // Second hash code of the file
  MPQ_INFO_FILE_SIZE       = 11;      // Original file size
  MPQ_INFO_COMPRESSED_SIZE = 12;      // Compressed file size
  MPQ_INFO_FLAGS           = 13;      // File flags
  MPQ_INFO_SEED            = 14;      // Encrypted file seed
  MPQ_INFO_UNFIXED_SEED    = 15;      // Unfixed seed of encrypted files
  MPQ_INFO_OFFSET          = 16;      // File offset in MPQ archive

  CCB_CHECKING_FILES       = 1;       // Checking archive : (dwParam1 = current, dwParam2 = total)
  CCB_COMPACTING_FILES     = 2;       // Compacting archive (dwParam1 = current, dwParam2 = total)
  CCB_CLOSING_ARCHIVE      = 3;       // Closing archive : No params used

  MPQLIB_VERSION  =  $01020101;
  MPQLIB_VER_STR  =  '1.2.1.1';       // Current version of MpqLib

type
  MPQ_FIND_DATA = record
    szFullPath:array [0..MAX_PATH - 1] of char;
    szFileName:PChar;
    dwFileSize:DWORD;
    dwCompSize:DWORD;
    dwFileFlags:DWORD;
    dwBlockIndex:DWORD;
    lcLocale:LCID;
  end;
  TCOMPACTCB = procedure(dwWorkType:DWORD; dwParam1:DWORD; dwParam2:DWORD; lpData:Pointer);stdcall;

// Archive opening/closing
function MpqOpenArchive(const szMpqName:PChar; dwPriority:DWORD; dwFlags:DWORD; var phMPQ:THANDLE; bNoListFile:BOOL = false):BOOL;stdcall;external 'MpqLib.dll' name 'A3';
{
  MpqOpenArchive - Open an archive, readonly
   return TRUE if successfully opened the archive, otherwise return FALSE.
    szMpqName  : MPQ Filename
    dwPriority : Priority for searching files in multiple archives, not used currently
    dwFlags    : Open flags, not used currently, please use MPQ_OPEN_NORMAL
    phMPQ      : Pointer to retrieve MPQ handle
    bNoListFile: Do not parse listfile for list on opening archive
}

function MpqCreateArchive(const szMpqName:PChar; dwCreationDisposition:DWORD; dwHashTableSize:DWORD; var phMPQ:THANDLE; dwBlockSize:DWORD = 3; bNoListFile:BOOL = false):BOOL;stdcall;external 'MpqLib.dll' name 'B4';
{
  MpqCreateArchive - Open or Create an archive
   return TRUE if successfully opened/created the archive, otherwise return FALSE.
    szMpqName            : MPQ Filename
    dwCreationDisposition: can be CREATE_NEW, CREATE_ALWAYS, OPEN_EXISTING or OPEN_ALWAYS, check Windows API CreateFile for more information
    dwHashTableSize      : Hashtable size, which indicates the maximum count of files allowed, range: 0x2-0x40000, should be power of 2
    phMPQ                : Pointer to retrieve MPQ handle
    dwBlockSize          : Block size of file compression, range: 0x01-0x0F, the real block size is (0x200 * 2 ^ n)
    bNoListFile          : Do not parse listfile for list on opening/creating archive and do not save it on closing
}

function MpqCloseArchive(hMPQ:THANDLE):BOOL;stdcall;external 'MpqLib.dll' name 'A4';
{
  MpqCloseArchive - Close an archive, works on handles retrieved by both MpqOpenArchive and MpqCreateArchive
   return TRUE if successfully closed the archive, otherwise return FALSE.
    hMPQ : handle of the MPQ archive to close
}

// File opening/closing
function MpqOpenFile(hMPQ:THANDLE; const szFileName:PChar; dwSearchScope:DWORD; var phFile:THANDLE):BOOL;stdcall;external 'MpqLib.dll' name 'A5';
{
  MpqOpenFile - Open a file
   return TRUE if successfully opened the file, otherwise return FALSE.
    hMPQ         : handle of the MPQ archive for opening file
    szFileName   : Filename or Hashtable Index(use typecast to convert a DWORD to const char *) of file to open
    dwSearchScope: MPQ_OPEN_INTERNAL - Open files in mpq archive
                   MPQ_OPEN_LOCAL    - Open files on local hdd
    phFile       : Pointer to retrieve file handle
}

function MpqCloseFile(hFile:THANDLE):BOOL;stdcall;external 'MpqLib.dll' name 'A6';
{
  MpqCloseFile - Close a file, works on handles retrieved by MpqOpenFile
   return TRUE if successfully closed the file, otherwise return FALSE.
    hFile : handle of the file to close
}

// File I/O
function MpqGetFileOffset(hFile:THANDLE; pdwFileOffsetHigh:PDWORD = nil):DWORD;stdcall;external 'MpqLib.dll' name 'A7';
{
  MpqGetFileOffset - Get file offset in the MPQ archive
   return the offset of the file, otherwise return 0xFFFFFFFF
    hFile            : File handle
    pdwFileOffsetHigh: Stores upper 32bits of the offset if valid
}

function MpqGetFileSize(hFile:THANDLE; pdwFileSizeHigh:PDWORD = nil):DWORD;stdcall;external 'MpqLib.dll' name 'A8';
{
  MpqGetFileSize - Get original file size
   return the original size of the file, otherwise return 0xFFFFFFFF
    hFile          : File handle
    pdwFileSizeHigh: Stores upper 32bits of the size if valid
}

function MpqSetFilePointer(hFile:THANDLE; lFileOffset:Longint; pdwFileOffsetHigh:PLongint; dwMethod:DWORD):DWORD;stdcall;external 'MpqLib.dll' name 'A9';
{
  MpqSetFilePointer - Move file stream pointer
   return the new offset of the pointer, otherwise return 0xFFFFFFFF
    hFile            : File handle
    lFileOffset      : The move offset
    pdwFileOffsetHigh: Stores upper 32bits of the new offset if valid
    dwMethod         : FIlE_BEGIN   - Move from beginning
                       FILE_CURRENT - Move from current position
                       FILE_END     - Move from end of file (lFileOffset should be equal or less than zero)
}

function MpqReadFile(hFile:THANDLE; lpBuffer:Pointer; dwToRead:DWORD; pdwRead:PDWORD = nil; lpOverlapped:POVERLAPPED = nil):BOOL;stdcall;external 'MpqLib.dll' name 'B1';
{
  MpqReadFile - Read data from a file
   return TRUE if successfully read data, otherwise return FALSE
    hFile       : File handle
    lpBuffer    : Buffer to store read data
    dwToRead    : Count of bytes to read
    pdwRead     : Stores count of read bytes if valid
    lpOverlapped: Check Windows API ReadFile for information, do not work on internal files of archive
}

function MpqExtractFile(hMpq:THANDLE; const szToExtract:PChar; const szExtracted:PChar):BOOL;stdcall;external 'MpqLib.dll' name 'B2';
{
  MpqExtractFile - Extract a file from the MPQ archive
   return TRUE if successfully extract the file, otherwise return FALSE
    hMpq       : MPQ handle
    szToExtract: Name of the file to extract
    szExtracted: Full path name of extracted file
}

function MpqAddListFile(hMpq:THANDLE; const szListFile:PChar):BOOL;stdcall;external 'MpqLib.dll' name 'B3';
{
  MpqAddListFile - Merge a external listfile into current MPQ archive
   return TRUE if successfully added the listfile, otherwise return FALSE
    hMpq       : MPQ handle
    szListFile : Full path name of the listfile
}

// Modify archive functions
function MpqAddFile(hMPQ:THANDLE; const szFileName:PChar; const szArchivedName:PChar; dwFlags:DWORD; var pbReplaced:BOOL):BOOL;stdcall;external 'MpqLib.dll' name 'B5';
{
  MpqAddFile - Add a file into the MPQ archive
   return TRUE if successfully added the file, otherwise return FALSE
    hMpq          : MPQ handle
    szFileName    : Name of file to add
    szArchivedName: Name of archived file
    dwFlags       : Flags of file (check constant part)
    pbReplaced    : If valid, it retrieve whether the file is replaced or not(newly added)
}

function MpqAddWave(hMPQ:THANDLE; const szFileName:PChar; const szArchivedName:PChar; dwFlags:DWORD; dwQuality:DWORD; var pbReplaced:BOOL):BOOL;stdcall;external 'MpqLib.dll' name 'B6';
{
  MpqAddWave - Add a wave file into the MPQ archive
   return TRUE if successfully added the wave file, otherwise return FALSE
    hMpq          : MPQ handle
    szFileName    : Name of file to add
    szArchivedName: Name of archived file
    dwFlags       : Flags of file (check constant part)
    dwQuality     : Wave quality (check constant part)
    pbReplaced    : If valid, it retrieve whether the file is replaced or not(newly added)
}

function MpqAddBuffer(hMPQ:THandle; pbData:PBYTE; dwLength:DWORD; const szArchivedName:PChar; dwFlags:DWORD; bIsWave:BOOL; var pbReplaced:BOOL):BOOL;stdcall;external 'MpqLib.dll' name 'C8';
{
  MpqAddBuffer - Add buffer data into the MPQ archive as a file
   return TRUE if successfully added the file, otherwise return FALSE
    hMpq          : MPQ handle
    pbData        : Data buffer
    dwLength      : Length of data buffer
    szArchivedName: Name of archived file
    dwFlags       : Flags of file (check constant part)
    bIsWave       : Indicates if the it is a wave file
    dwQuality     : Wave quality (check constant part)
    pbReplaced    : If valid, it retrieve whether the file is replaced or not(newly added)
}

function MpqRemoveFile(hMPQ:THANDLE; const szFileName:PChar):BOOL;stdcall;external 'MpqLib.dll' name 'B7';
{
  MpqRemoveFile - Remove a file from the MPQ archive
   return TRUE if successfully removed the file, otherwise return FALSE
    hMPQ      : MPQ handle
    szFileName: Name of file to remove
}

function MpqRenameFile(hMPQ:THANDLE; const szOldFileName:PChar; const szNewFileName:PChar):BOOL;stdcall;external 'MpqLib.dll' name 'B8';
{
  MpqRemoveFile - Rename a file in the MPQ archive
   return TRUE if successfully renamed the file, otherwise return FALSE
    hMPQ         : MPQ handle
    szOldFileName: Name of file to rename
    szNewFileName: Name of renamed file
}

// Retrieving archive/file info functions
function MpqHasFile(hMPQ:THANDLE; szFileName:PChar):BOOL;stdcall;external 'MpqLib.dll' name 'C1';
{
  MpqHasFile - Check if a file exists in the MPQ archive
   return TRUE if the file exists, otherwise return FALSE
    hMPQ      : MPQ handle
    szFileName: Name of file to check
}

function MpqGetFileName(hFile:THANDLE; szFileName:PChar):BOOL;stdcall;external 'MpqLib.dll' name 'C2';
{
  MpqHasFile - Retrieve filename from a file handle
   return TRUE if the successfully retrieved, otherwise return FALSE
    hFile     : File handle
    szFileName: Buffer to retrieve the filename, the size should be enough to store the string (always be MAX_PATH)
}

function MpqGetFileInfo(hMpqOrFile:THANDLE; dwInfoType:DWORD):DWORD;stdcall;external 'MpqLib.dll' name 'C3';
{
  MpqGetFileInfo - Retrieved archive/file info from handle
   return info data if successfully retrieved, otherwise return 0xFFFFFFFF
    hHandle   : MPQ or File handle
    dwInfoType: Information type (check constant part)
}

// Find file functions
function MpqFindFirstFile(hMPQ:THANDLE; const szMask:PChar; var lpFindFileData:MPQ_FIND_DATA; const szListFile:PChar):THANDLE;stdcall;external 'MpqLib.dll' name 'C4';
{
  MpqFileFirstFile - Find first matching file in the MPQ archive
   return search handle if successfully find a matching file, otherwise return NULL
    hMPQ          : MPQ handle
    szMask        : Matching mask for searching
    lpFindFileData: Retrieve info of found file
    szListFile    : Additional listfile for searching
}

function MpqFindNextFile(hFind:THANDLE; var lpFindFileData:MPQ_FIND_DATA):BOOL;stdcall;external 'MpqLib.dll' name 'C5';
{
  MpqFileNextFile - Find next matching file in the MPQ archive
   return TRUE if successfully find a matching file, otherwise return FALSE
    hFind         : Search handle
    lpFindFileData: Retrieve info of found file
}

function MpqFindClose(hFind:THANDLE):BOOL;stdcall;external 'MpqLib.dll' name 'C6';
{
  MpqFileClose - Close a search handle
   return TRUE if successfully closed the handle, otherwise return FALSE
    hFind : Search handle to close
}

// Compact/Flush function
function MpqCompactArchive(hMPQ:THANDLE; const szFileList:PChar = nil; CompactCB:TCOMPACTCB = nil; lpCallbackData:Pointer = nil):BOOL;stdcall;external 'MpqLib.dll' name 'C7';
{
  MpqCompactArchive - Compact/Flush an archive
   return TRUE if successfully compacting the archive, otherwise return FALSE
    hMPQ          : MPQ handle
    szListFile    : Additional listfile for compacting
    CompactCB     : Compact callback function
    lpCallBackData: lpData parameter of callback function
}

// Locale functions
function MpqSetLocale(lcNewLocale:LCID):LCID;stdcall;external 'MpqLib.dll' name 'A1';
{
  MpqSetLocale - Set current locale of the archive (will affect both read and modify functions)
   return old locale ID
    lcNewLocale : New Locale ID
}

function MpqGetLocale():LCID;stdcall;external 'MpqLib.dll' name 'A2';
{
  MpqSetLocale - retrieve current locale ID
   return current locale ID
}

function MpqSetFileLocale(hFile:THANDLE; lcNewLocale:LCID):BOOL;stdcall;external 'MpqLib.dll' name 'B9';
{
  MpqSetLocale - Set locale of a file (only affect files in MPQ archives)
   return TRUE if successfully set the locale, otherwise return FALSE
    hFile       : File handle
    lcNewLocale : New Locale ID
}

implementation

end.
